@@ -107,6 +107,8 @@ class Esme(object):
 
         try:
             raw_len = self._socket.recv(4)
+            while len(raw_len) < 4:
+                raw_len += self._socket.recv(4 - len(raw_len))
         except socket.timeout:
             raise
         except socket.error as e:
@@ -180,17 +182,6 @@ class Esme(object):
             logging.info('Unbind command received')
             resp = smpp.make_pdu('unbind_resp', client=self)
             self.send_pdu(resp)
-        elif p.command in ['bind_transmitter', 'bind_receiver', 'bind_transceiver']:
-            if not self.bind_handler(p):
-                self.state = consts.SMPP_CLIENT_STATE_CLOSED
-                try:
-                    self._socket.close()
-                except:
-                    pass
-                try:
-                    self._socket.shutdown(socket.SHUT_RDWR)
-                except:
-                    pass
         elif p.command == 'unbind_resp':
             self.state = consts.SMPP_CLIENT_STATE_CLOSED
             try:
@@ -255,7 +246,13 @@ class Esme(object):
         while self.state != consts.SMPP_CLIENT_STATE_CLOSED:
             try:
                 p = self.queue.get(timeout=1)
+
+                logging.debug('Get new task from queue: {command} ({uid})'.format(command=p.command,
+                                                                                  uid=self.uid))
                 self._request_handler(p)
+
+                logging.debug('Finished processing task from queue: {command} ({uid})'.format(command=p.command,
+                                                                                              uid=self.uid))
                 self.queue.task_done()
             except Empty:
                 pass
@@ -281,7 +278,19 @@ class Esme(object):
                                                                    error=consts.DESCRIPTIONS.get(p.status, 'unknown')),
                             int(p.status))
 
-                    self.queue.put(p)
+                    if p.command in ['bind_transmitter', 'bind_receiver', 'bind_transceiver']:
+                        if not self.bind_handler(p):
+                            self.state = consts.SMPP_CLIENT_STATE_CLOSED
+                            try:
+                                self._socket.close()
+                            except:
+                                pass
+                            try:
+                                self._socket.shutdown(socket.SHUT_RDWR)
+                            except:
+                                pass
+                    else:
+                        self.queue.put(p)
 
                 except exceptions.PDUError as e:
                     if ignore_error_codes and len(e.args) > 1 and e.args[1] in ignore_error_codes:
